       PROGRAM COMPARE
       USE numerical_libraries
 
C****************************************************************
C     FORTRAN77 CODE
C
C     THIS PROGRAM TESTS THE H0-HYPOTHESIS THAT 2 TIME SERIES
C     REALIZATIONS OF THE SAME PROCESS. IT BUILDS UP PHASE
C     SPACE AND CALCULATES Q^ AND VAR(Q^). 
C     FOR IDENTICAL PROCESSES S = Q^/ SQRT(VAR(Q^)) HAS MEAN
C     0 AND STANDARD DEVIATION 1.
C     FOR DIFFERENT PROCESSES LARGER VALUES OF S ARE 
C     EXPECTED.
C
C     WITH LOOP FOR SIMULATED DATA
C
C     Reference: Diks, van Zwet, Takens, DeGoede (1996).
C        Detecting differences between delay vector distributions.
C        Physical Review E, 53(3), 2169-2176.
C****************************************************************
      INTEGER I,J,M,N1,N2,NTAU,NDIM,L,TERM3,II,JJ,T2,T3,T4,T5,T10,T11,
     * SEED, TOT
     
      REAL X(500),Y(500),TAVG,TVAR,VARX,VARY,D,DN,VARQ,XAVG,
     * SIGMA,Q,QXX,QYY,QXY,EG,EGG,EHH,EH,G(1000),HIJ,MM1,MM2,
     * HLP,TMP,T1,TM,DIS,
     * QXXTOT(10000),QYYTOT(10000),QXYTOT(10000),
     * QTOT(10000),QSIGMA(10000),SIGMATOT(10000),
     * MQ,MQXX,MQYY,MQXY,MQSIGMA,MSIGMA,
     * SDQ,SDQXX,SDQYY,SDQXY,SDSIGMA,SDQSIMA,TEST      
     
      PARAMETER(NOBS1=100,NOBS2=NOBS1,NREP=1000)
      
      NTAU=1  ! DELAY
      NDIM=3  ! EMBEDDING DIM.
      L=10    ! BLOCK LENGTH
      D=0.5   ! BANDWIDTH
      
      OPEN(2,FILE='C:\CDIKS\CDIKS.OUT')   !OUTPUT
      WRITE(2,*) ' PROGRAM: DIKS4.FOR '
      WRITE(2,*) 
      WRITE(2,*) ' THE PROGRAM TESTS THE H0-HYPOTHESIS THAT TIME SERIES'
      WRITE(2,*) ' REALIZATIONS OF THE SAME PROCESS.IT BUILDS UP PHASE '
      WRITE(2,*) ' SPACE AND CALCULATES Q^ AND VAR(Q^). ' 
      WRITE(2,*) ' FOR IDENTICAL PROCESSES S = Q^/ SQRT(VAR(Q^)) '
      WRITE(2,*) ' HAS MEAN 0 AND STANDARD DEVIATION 1. '
      WRITE(2,*) ' OTHERWISE LARGER VALUES OF S ARE EXPECTED'
      WRITE(2,*)
      WRITE(2,*) ' NTAU (DELAY) = ', NTAU
      WRITE(2,*) ' NDIM (EMBEDDING DIM) =', NDIM
      WRITE(2,*) ' BLOCK LENGTH = ', L
      WRITE(2,*) ' BANDWIDTH = ', D
      WRITE(2,*) ' NOBS1  = ', NOBS1, ' NOBS2 = ', NOBS2 
      WRITE(2,*) ' NUMBER OF REPLICATIONS = ', NREP
      WRITE(2,*)
      WRITE(2,*) ' MEAN VALUES (AVERAGED OVER NREP REPL) TEST STATs:' 
      
      TOT=0.0    
      DO 9999  UU=1,NREP  ! BIG LOOP FOR REPLICATIONS
      WRITE(*,*) 'NREP = ', UU
      
      SEED=12345+UU
      CALL RNSET(SEED)
      CALL RNUN(NOBS1,X) 
      CALL RNUN(NOBS2,Y)

      N1=NOBS1
      N2=NOBS2
     
      XAVG=0.0
      VARX=0.0
      DO I=1,N1
         XAVG=XAVG+X(I)
         VARX=VARX+X(I)*X(I)
      ENDDO
     
      DO I=1,N2
         XAVG=XAVG+Y(I)
         VARX=VARX+Y(I)*Y(I)
      ENDDO

      TAVG=XAVG/FLOAT(N1+N2)
      TVAR=VARX/FLOAT(N1+N2)

      TERM1=(FLOAT(N1+N2)/FLOAT(N1+N2-1))
      TM=3*(TVAR-(TAVG*TAVG))
      TERM2=2.0*SQRT(TM)
      
      DO I=1,N1
         X(I)=SQRT(TERM1) *(X(I)-TAVG)/TERM2
      ENDDO
      DO I=1,N2
         Y(I)=SQRT(TERM1) *(Y(I)-TAVG)/TERM2
      ENDDO

C     SWITCH TO VECTORS

      N1=N1-(NDIM-1)*NTAU
    
      MM1=FLOAT(N1/L)
      
      N2=N2-(NDIM-1)*NTAU
      N1=INT(MM1)
      
      MM2=FLOAT(N2/L)
      N2=INT(MM2)
      DN=2*D
      
C     INITIALIZE CUMULATIVE VARIABLES      

      Q=0.0
      QXX=0.0
      QYY=0.0
      QXY=0.0
      
      EHH=0.0
      EH=0.0
      EG=0.0
      EGG=0.0
      TMP=0.0
      
      DO 99 I=0,(N1+N2-1)
            G(I+1)=0.0
99    CONTINUE

      DO 100 I=1,(N1+N2-1)
            DO 98 J=0,I-1
                  TMP=0.0
                  HIJ=0.0
                  IF((I.LT.N1).AND.(J.LT.N1)) THEN
                     DO 97 II=I*L,(I+1)*L-1
                        DO 96 JJ=J*L,(J+1)*L-1
                           DIS=0.0
                           DO 95 M=0,NDIM-1
                              TMP=X(II+M*NTAU+1)-X(JJ+M*NTAU+1)
                              DIS=DIS+TMP*TMP
95                         CONTINUE
                           T1=-DIS/(DN*DN)
                           TMP=EXP(T1)
                           T2=L*L
                           HIJ=HIJ+TMP/FLOAT(T2)
96                      CONTINUE
97                   CONTINUE
                     QXX=QXX+HIJ
                  ENDIF
                 
                  IF ((I.GE.N1).AND.(J.LT.N1)) THEN
                      DO 94 II=I*L,(I+1)*L-1
                         DO 93 JJ=J*L,(J+1)*L-1
                            DIS=0.0
                            DO 92 M=0,NDIM-1
                               TMP=Y(II-N1*L+M*NTAU+1)-X(JJ+M*NTAU+1)
                               DIS=DIS+TMP*TMP
92                          CONTINUE
                            T1=-DIS/(DN*DN)
                            TMP=EXP(T1)
                            T2=L*L
                            HIJ=HIJ+TMP/FLOAT(T2) 
93                       CONTINUE
94                    CONTINUE
                      QXY=QXY+HIJ
                 ENDIF

                 IF((I.GE.N1).AND.(J.GE.N1)) THEN
                      DO 91 II=I*L,(I+1)*L-1
                         DO 90 JJ=J*L,(J+1)*L-1
                            DIS=0.0
                            DO 89 M=0,NDIM-1
                             TMP=Y(II-N1*L+M*NTAU+1)-Y(JJ-N1*L+M*NTAU+1)
                               DIS=DIS+TMP*TMP
89                          CONTINUE
                            T1=-DIS/(DN*DN)
                            TMP=EXP(T1)
                            T2=L*L
                            HIJ=HIJ+TMP/FLOAT(T2)
90                       CONTINUE
91                    CONTINUE
                      QYY=QYY+HIJ
                 ENDIF
                   
                 G(I+1)=G(I+1)+HIJ
                 G(J+1)=G(J+1)+HIJ
                   
                 EHH=EHH+HIJ*HIJ
                 EH=EH+HIJ
98         CONTINUE
100   CONTINUE

      TERM3 = 0.5*(N1+N2)*(N1+N2-1)
      EH=EH/FLOAT(TERM3)

      T10=N1+N2-2            
      DO I=0,N1+N2-1
         G(I+1)=G(I+1)/FLOAT(T10)
      ENDDO

C     DETERMINE THE AVERAGE EG OF THE GI'S 

      DO I=1,(N1+N2)
         EG=EG+G(I)
      ENDDO
      EG = EG/FLOAT(N1+N2)
      
C     CENTER THE G(I)'S AND DETERMINE THEIR MEAN SQUARE EGG
      
C      WRITE(2,*)
      DO I=1,(N1+N2)
         G(I)=G(I)-EG
         EGG=EGG+G(I)*G(I)
      ENDDO

      T11=N1+N2      
      EGG=EGG/FLOAT(T11)

C     NORMALIZE THE MEAN SQUARE OF THE H'S AND THE Q'S FOUND
      
      EHH = EHH/FLOAT(TERM3)

      T3=N1*(N1-1)
      T4=N2*(N2-1)
      T5=N1*N2
      
      QXX=2*QXX/FLOAT(T3)
      QYY=2*QYY/FLOAT(T4)
      QXY=QXY/FLOAT(T5)
      
      Q=QXX+QYY-2.0*QXY
      
      QXXTOT(UU)=QXX
      QYYTOT(UU)=QYY
      QXYTOT(UU)=QXY
      QTOT(UU)=Q
 
C     DETERMINE THE CONDITIONAL VARIANCE

      TERM4=(2.0*(N1+N2-2)*EGG/FLOAT(N1+N2-1))
      VARQ=EHH-EH*EH-TERM4
      
      HLP=2*(N1+N2-1)*(N1+N2-1)*(N1+N2-2)
      HLP=HLP/FLOAT(N1+N2-3)
      
      HLP=HLP/FLOAT(N1)
      HLP=HLP/FLOAT(N2)
      HLP=HLP/FLOAT(N1-1)
      HLP=HLP/FLOAT(N2-1)
      
      VARQ = VARQ*HLP    
      SIGMA=SQRT(VARQ)
      
      SIGMATOT(UU)=SIGMA
      QSIGMA(UU)=Q/SIGMA
      TEST=Q/SIGMA
      
C     3-SIGMA RULE

      IF(TEST.GT.3) THEN 
	TOT=TOT+1
	ELSE
      ENDIF

cc    WRITE(2,*) ' Q = ', Q 
cc    WRITE(2,*) ' QXX = ', QXX     
cc    WRITE(2,*) ' QYY = ', QYY
cc    WRITE(2,*) ' QXY = ', QXY  
cc    WRITE(2,*) ' SIGMA = ', SIGMA
      WRITE(2,*)  Q/SIGMA, UU
CC    WRITE(2,*)

9999  CONTINUE  ! END BIG LOOP REPLICATION UU=1,NREP

      MQ=0.0
      MSIGMA=0.0
      MQSIGMA=0.0
      MQXX=0.0
      MQYY=0.0
      MQXY=0.0
      
      DO UU=1,NREP
         MQ=MQ+QTOT(UU)
         MQXX=MQXX+QXXTOT(UU)
         MQYY=MQYY+QYYTOT(UU)
         MQXY=MQXY+QXYTOT(UU)
         MQSIGMA=MQSIGMA+QSIGMA(UU)
         MSIGMA=MSIGMA+SIGMATOT(UU)
      ENDDO
      
      MQ=MQ/FLOAT(NREP)
      MQXX=MQXX/FLOAT(NREP)
      MQYY=MQYY/FLOAT(NREP)
      MQXY=MQXY/FLOAT(NREP)
      MQSIGMA=MQSIGMA/FLOAT(NREP)  ! Q/SIGMA
      MSIGMA=MSIGMA/FLOAT(NREP)    ! SIGMA  
      
      SDQ=0.0
      SDQXX=0.0
      SDQXY=0.0
      SDQYY=0.0
      SDQSIGMA=0.0
      SDSIGMA=0.0
      
      DO UU=1,NREP
         SDQ=SDQ+(QTOT(UU) - MQ)**2
         SDQXX=SDQXX+(QXXTOT(UU)-MQXX)**2
         SDQXY=SDQXY+(QXYTOT(UU)-MQXY)**2
         SDQYY=SDQYY+(QYYTOT(UU)-MQYY)**2
         SDQSIGMA=SDQSIGMA+(QSIGMA(UU)-MQSIGMA)**2
         SDSIGMA=SDSIGMA+(SIGMATOT(UU)-MSIGMA)**2
      ENDDO
         SDQ=SDQ/FLOAT(NREP-1)
         SDQXX=SDQXX/FLOAT(NREP-1)
         SDQXY=SDQXY/FLOAT(NREP-1)
         SDQYY=SDQYY/FLOAT(NREP-1)
         SDQSIGMA=SDQSIGMA/FLOAT(NREP-1)
         SDSIGMA=SDSIGMA/FLOAT(NREP-1)
         
     
      WRITE(2,*)
      WRITE(2,*) ' Q = ', MQ, ' SQRT(Q) = ',SQRT(SDQ)
      WRITE(2,*) ' QXX = ', MQXX, ' SQRT(QXX) = ', SQRT(SDQXX)
      WRITE(2,*) ' QYY = ', MQYY, ' SQRT(QYY) = ', SQRT(SDQYY)
      WRITE(2,*) ' QXY = ', MQXY, ' SQRT(QXY) = ', SQRT(SDQXY)
      WRITE(2,*)
      WRITE(2,*) ' S = (Q/SIGMA) = ',MQSIGMA,' SQRT(S) =',SQRT(SDQSIGMA)
      WRITE(2,*) ' SIGMA = ', MSIGMA , ' SQRT(SIGMA) = ', SQRT(SDSIGMA) 
      WRITE(2,*)
      WRITE(2,*) ' TEST STATISTICS: Q, QXX, QXY, QYY  AND SDs' 
      WRITE(2,77)  MQ, SQRT(SDQ) 
      WRITE(2,77)  MQXX, SQRT(SDQXX)
      WRITE(2,77)  MQYY, SQRT(SDQYY)
      WRITE(2,77)  MQXY, SQRT(SDQXY)
      WRITE(2,*)
      WRITE(2,*) ' TEST STATISTIC S, SIGMA AND SDs '
      WRITE(2,77)  MQSIGMA,SQRT(SDQSIGMA)
      WRITE(2,77)  MSIGMA, SQRT(SDSIGMA)
77    FORMAT(2F10.6)
      WRITE(2,*) ' TOTAL NUMBER OF S-VALUES > 3:  ', TOT

      STOP
      END
      
